from trac.core import *
from trac.resource import Resource
from trac.web.api import ITemplateStreamFilter

from genshi.builder import tag
from genshi.filters.transform import Transformer

from tractags.api import DefaultTagProvider, TagSystem
from tracdiscussion.api import DiscussionApi, ITopicChangeListener 

class DiscussionTopicTagProvider(DefaultTagProvider):
    """Tag provider for the discussion topics."""
    
    #  Tags plugin doesn't seem to support realms with "-" in them :(
    #realm = 'discussion-topic'  
    realm = 'discussiontopic'  

    def check_permission(self, perm, operation):
        #map = {'view': 'DISCUSSION_VIEW', 'modify': 'DISCUSSION_MODIFY'}
        map = {'view': 'WIKI_VIEW', 'modify': 'WIKI_MODIFY'}
        return super(DiscussionTopicTagProvider, self).check_permission(perm, operation) \
            and map[operation] in perm

    def describe_tagged_resource(self, req, resource):
        pass
        #if not self.check_permission(req.perm(resource), 'view'):
        #    return ''
        #api = self.env[DiscussionApi]
        #real_resource = Resource('discussion-topic', resource.id)
        #return api.get_resource_description(real_resource, 'summary')

class DiscussionTopicTagInterface(Component):
    """Implement the user interface for tagging discussion topics."""
    implements(ITemplateStreamFilter, 
               ITopicChangeListener)
    
    #realm = 'discussion-topic'
    realm = 'discussiontopic'

    # ITemplateStreamFilter methods
    def filter_stream(self, req, method, filename, stream, data):
        self.log.debug('filtering stream "%s"' % (filename,))
        if filename == 'topic-add.html':
            return self._topic_add(req, stream)
        return stream
 
    # ITopicChangeListener methods
    def topic_created(self, context, topic):
        resource = Resource(self.realm, topic['id'])
        self._update_tags(context.req, resource)

    def topic_changed(self, context, topic, old_topic):
        resource = Resource(self.realm, topic['id'])
        self._update_tags(context.req, resource)

    def topic_deleted(self, context, topic):
        resource = Resource(self.realm, topic['id'])
        self._delete_tags(context.req, resource)

    # Internal methods
    def _delete_tags(self, req, resource):
        tag_system = TagSystem(self.env)
        tag_system.delete_tags(req, resource)
              
    def _update_tags(self, req, resource):
        tag_system = TagSystem(self.env)

        newtags = tag_system.split_into_tags(req.args.get('tags', ''))
        oldtags = tag_system.get_tags(req, resource)

        if oldtags != newtags:
            tag_system.set_tags(req, resource, newtags)
            return True
        return False

    def _topic_add(self, req, stream):
        insert = tag.div(class_='field')(
            tag.label(
                tag("Tag under: ("),
                tag.a("view all tags", href=req.href.tags()),
                tag(")"),
                tag.br(),
                tag.input(id='tags', type='text', name='tags', size='50',),
                )
            )
        return stream | Transformer('//div[@class="subscribers"]').append(insert)
